require "spec_helper"

describe Metamorphosis::Engine do

  let!(:engine){ Metamorphosis::Engine.instance }

  before{ Metamorphosis.register_predefined_units }
  after { Metamorphosis.reset! }

  describe "#numeric_to_unit_value" do
    it "Creates a UnitValue from a numeric value and a unit name" do
      engine.numeric_to_unit_value(42.0, 'J').
        should == Metamorphosis::UnitValue.new(:J, 42.0)
    end

    it "Creates a UnitValue from a numeric value and a unit name with a prefix" do
      val = engine.numeric_to_unit_value(42.0, 'kJ')
      val.should == Metamorphosis::UnitValue.new(:J, 42.0, 1_000)
      val.to_f.should == 42_000
    end

    it "returns nil when the unit_name is not recognized" do
      engine.numeric_to_unit_value(1.0, 'junk').should be_nil
    end
  end

  describe "#convert_to" do
    it "converts 16 ounces to cups" do
      engine.convert_to(:cups, 16.ounces).to_f.
        should be_within(0.000001).of(2)
    end

    it "converts 73 degress F to celsius" do
      engine.convert_to(:celsius, 73.degree_fahrenheit).to_f.
        should be_within(0.0001).of(22.7778)
    end

    it "converts btu's to kilowatt_hour" do
      engine.convert_to(:kilowatt_hours, 1054.350.british_thermal_units).
        to_f.should be_within(0.000001).of(0.3087927)
    end

    it "convert joules's to therms" do
      engine.convert_to(:therms, 1.J).to_f.
        should be_within(0.0001).of(1/1.05506e+8)
    end

    it "converts to the same unit" do
      engine.convert_to(:therms, 44.therms).to_f.
        should == 44
    end
  end

  describe "#to_base_value" do
    it "converts units into base unit values" do
      engine.to_base_value(:energy, 1.british_thermal_units).
        should == 1_054.35
    end

    it "uses the first proc to convert to the base value when provided" do
      engine.to_base_value(:temperature, 1.3.celsius).
        should == 274.45
    end
  end

  describe "#from_base_values" do
    it "converts from base values" do
      engine.from_base_value(:energy, :british_thermal_units, 1_054.35, 1.0).
        should == 1.british_thermal_units
    end

    it "converts from base values using the last proc when provided" do
      engine.from_base_value(:temperature, :celsius, 274.45, 1.0).to_f.
        should be_within(0.00000000001).of(1.3)
    end
  end

  # Incompatible type checking (i.e. area to volume
  it "raises an Error when the unit types are not compatible" do
    expect{ engine.convert_to(:square_meter, 16.ounces) }.
      to raise_error(Metamorphosis::IncompatibleUnitsError)
  end

  it "will register multiple units symbosl to the same value" do
    engine.register_unit(:stuff, [:value1a, :value1b], 1.0)
    engine.register_unit(:stuff, [:value2c, :value2d], 2.0)

    1.value1b.value1a.to_f.should == 1.value1a.to_f
    1.value1a.should == 1.value1b
    1.value2c.value1a.to_f.should == 2.0
    1.value2c.should == 2.value1a
    1.value2c.should == 1.value2d
    1.value2d.should == 2.value1b
  end

  it "#register_units allows units to be defined based on other units" do
    Metamorphosis.reset!
    Metamorphosis.register_unit(:energy, :btu, 1.0)
    Metamorphosis.register_unit(:energy, :therms, 100_000.btu)
    Metamorphosis.register_unit(:energy, :kwh, 0.03413.therms)
    1.kwh.to.btu.to_f.should == 3413.0
  end

end
